<?php

namespace Drupal\cacheprofiler;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Service to get the cache metadata of an entity.
 */
class CacheMetadataExtractor implements CacheMetadataExtractorInterface {

  /**
   * Cache tags.
   *
   * @var array<string>
   */
  protected $cacheTags = [];

  /**
   * Entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * CacheMetadataExtractor constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   Entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager) {
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * {@inheritdoc}
   */
  public function extractCacheMetadata($entity, $displayId = 'default') {
    $this->cacheTags = [];
    $this->storeEntityCacheTags($entity);
    $this->getReferencedEntitiesCacheTags($entity, $displayId);
    $this->cacheTags = array_unique($this->cacheTags);
    $cache_tags = $this->cacheTags;
    $this->cacheTags = [];
    return $cache_tags;
  }

  /**
   * Store the cache tags of the entity.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   Content entity.
   */
  protected function storeEntityCacheTags(ContentEntityInterface $entity) {
    $this->cacheTags = array_merge($this->cacheTags, $entity->getCacheTags());
  }

  /**
   * Get the cache tags of the referenced entities.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   Content entity.
   * @param string $displayId
   *   Display ID.
   */
  protected function getReferencedEntitiesCacheTags(ContentEntityInterface $entity, $displayId = 'default') {
    $fields = $this->getFieldsForDisplay($entity, $displayId);

    foreach ($fields as $field_id => $field_display_config) {
      if (!$entity->hasField($field_id)) {
        continue;
      }
      $field = $entity->get($field_id);
      $field_type = $field->getFieldDefinition()->getType();

      // Check if the field is an entity reference field.
      if (!in_array($field_type, ['entity_reference', 'entity_reference_revisions'])) {
        continue;
      }

      $field_display = $field_display_config['settings']['display'] ?? 'default';

      $referencedEntities = $field->referencedEntities();
      foreach ($referencedEntities as $referencedEntity) {
        $this->storeEntityCacheTags($referencedEntity);
        // Recursive call to get the cache tags of the referenced entities.
        $this->getReferencedEntitiesCacheTags($referencedEntity, $field_display);
      }
    }
  }

  /**
   * Get the fields for a display.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   Content entity.
   * @param string $displayId
   *   Display ID.
   *
   * @return array
   *   Fields for display.
   */
  protected function getFieldsForDisplay(ContentEntityInterface $entity, string $displayId): array {
    $entity_display_view = $entity->getEntityTypeId() . '.' . $entity->bundle() . '.' . $displayId;
    $viewDisplay = $this->entityTypeManager->getStorage('entity_view_display')->load($entity_display_view);
    if ($viewDisplay === NULL) {
      return [];
    }
    $fields = $viewDisplay->get('content') ?? [];
    return $fields;
  }

}
