<?php

declare(strict_types=1);

namespace Drupal\cacheprofiler\DataCollector;

use Drupal\cacheprofiler\CacheMetadataExtractor;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\node\NodeInterface;
use Drupal\webprofiler\DataCollector\DataCollector;
use Drupal\webprofiler\DataCollector\HasPanelInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\DataCollector\LateDataCollectorInterface;
use Symfony\Component\VarDumper\Cloner\Data;

/**
 * Collects Cache data.
 */
class CacheProfilerDataCollector extends DataCollector implements CacheProfilerDataCollectorInterface, LateDataCollectorInterface, HasPanelInterface {

  use StringTranslationTrait;

  /**
   * The cache metadata extractor.
   *
   * @var \Drupal\cacheprofiler\CacheMetadataExtractorInterface
   */
  protected $cacheMetadataExtractor;

  /**
   * CacheProfiler constructor.
   */
  public function __construct(CacheMetadataExtractor $cacheMetadataExtractor) {
    $this->reset();
    $this->cacheMetadataExtractor = $cacheMetadataExtractor;
  }

  /**
   * {@inheritdoc}
   */
  public function collect(Request $request, Response $response, \Throwable $exception = NULL): void {
    // I want to get the route of this request included the node id.
    if ($request->attributes->get('_route') !== 'entity.node.canonical') {
      return;
    }

    $node = $request->attributes->get('node');
    if (!$node instanceof NodeInterface) {
      return;
    }

    // Collect the data related to cache tags from elements.
    // Likely, the view mode for the node is 'full'.
    $elements_cache_tags = $this->cacheMetadataExtractor->extractCacheMetadata($node, 'full');

    // Collect the data related to cache tags from response headers.
    if (method_exists($response, 'getCacheableMetadata')) {
      $cacheable_metadada = $response->getCacheableMetadata();
      $response_cache_tags = $cacheable_metadada->getCacheTags() ?? [];
      $not_propagated_cache_tags = array_diff($elements_cache_tags, $response_cache_tags);
      $this->data['count_not_propagated_cache_tags'] = count($not_propagated_cache_tags);
      $this->data['not_propagated_cache_tags'] = $not_propagated_cache_tags;
    }

  }

  /**
   * {@inheritdoc}
   */
  public function reset() {
    $this->data = ['count_not_propagated_cache_tags' => 0, 'not_propagated_cache_tags' => []];
  }

  /**
   * Get the count of cache tags that are not propagated.
   *
   * @return int
   *   Count.
   */
  public function getCountNotPropagedCacheTags() {
    $values = ($this->data instanceof Data) ? $this->data->getValue(TRUE) : $this->data;
    return (int) ($values['count_not_propagated_cache_tags'] ?? 0);
  }

  /**
   * Get the cache tags that are not propagated.
   *
   * @return array
   *   The cache tags.
   */
  public function getNotPropagedCacheTags(): array {
    /** @var array<mixed> $values */
    $values = ($this->data instanceof Data) ? $this->data->getValue(TRUE) : $this->data;
    return $values['not_propagated_cache_tags'] ?? [];
  }

  /**
   * {@inheritdoc}
   */
  public function getName() {
    return 'cacheprofiler';
  }

  /**
   * {@inheritdoc}
   */
  public function lateCollect() {
    $this->data = $this->cloneVar($this->data);
  }

  /**
   * {@inheritdoc}
   */
  public function getPanel(): array {
    $tabs = [];
    $tabs[] = [
      'label' => "Not Propagated Cache Tags",
      'content' => $this->renderTableNotPropagated(),
    ];

    return [
      '#theme' => 'webprofiler_dashboard_tabs',
      '#tabs' => $tabs,
    ];
  }

  /**
   * Create table with the results.
   */
  public function renderTableNotPropagated() {
    $rows = [];
    foreach ($this->getNotPropagedCacheTags() as $cache_tag) {
      $rows[] = [$cache_tag];
    }
    return [
      '#theme' => 'webprofiler_dashboard_section',
      '#data' => [
        '#type' => 'table',
        '#header' => [
          $this->t('Cache tag'),
        ],
        '#rows' => $rows,
        '#attributes' => [
          'class' => [
            'webprofiler__table',
          ],
        ],
      ],
    ];

  }

}
